/*
 * Written by Mark Spencer <markster@digium.com>
 * Based on previous works, designs, and architectures conceived and
 * written by Jim Dixon <jim@lambdatel.com>.
 *
 * Copyright (C) 2001 Jim Dixon / Zapata Telephony.
 * Copyright (C) 2001-2008 Digium, Inc.
 *
 * All rights reserved.
 *
 * Primary Author: Mark Spencer <markster@digium.com>
 * Radio Support by Jim Dixon <jim@lambdatel.com>
 */

/*
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2 as published by the
 * Free Software Foundation. See the LICENSE file included with
 * this program for more details.
 */

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>

#include <dahdi/user.h>
#include "dahdi_tools_version.h"

#define BLOCK_SIZE DAHDI_DEFAULT_BLOCKSIZE 

void print_packet(unsigned char *buf, int len)
{
	int x;
	printf("{\n\t");
	for (x=0;x<len;x++) {
		printf("%02x ",buf[x]);

		if (!((x+1)%16))
			printf("\n\t");
	}
	printf("\n}\n");
}

unsigned char sign_packet(unsigned char *buf, int len)
{
	if (len < 4)
		return 0x00;

	*buf++ = 0x55;
        *buf++ = 0xAA;
        *buf++ = 0x55;
        *buf++ = 0xAA;

	return 0x01;
}

int search_signature(unsigned char *buf, int len)
{
	int x;
	unsigned char found = 0;

	for (x=0; (x<(len-4)) && !found; x++) {
		if ((buf[x]==0x55) && (buf[x+1]==0xAA) && (buf[x+2]==0x55) && (buf[x+3]==0xAA))
			found++;
	}

	if (found)
		return (x+3);
	
	return -1;
}



int main(int argc, char *argv[])
{
	int fd;
	int res, x;
	int i;
	struct dahdi_params tp;
	int bs = BLOCK_SIZE;
	int skipcount = 0;
	int startbyte = 0;
	int packets = 0;
	unsigned char signaturefound = 0;
	unsigned char c=0,c1=0;
	unsigned char inbuf[BLOCK_SIZE];
	unsigned char outbuf[BLOCK_SIZE];
	int setup=0;
	int errors=0;
	int bytes=0;
	int timeout=0;
	time_t start_time=0;
	if (argc < 2 || argc > 3 ) {
		fprintf(stderr, "Usage: %s <DAHDI device> [timeout]\n",argv[0]);
		exit(1);
	}
	fd = open(argv[1], O_RDWR, 0600);
	if (fd < 0) {
		fprintf(stderr, "Unable to open %s: %s\n", argv[1], strerror(errno));
		exit(1);
	}
	if (ioctl(fd, DAHDI_SET_BLOCKSIZE, &bs)) {
		fprintf(stderr, "Unable to set block size to %d: %s\n", bs, strerror(errno));
		exit(1);
	}
	if (ioctl(fd, DAHDI_GET_PARAMS, &tp)) {
		fprintf(stderr, "Unable to get channel parameters\n");
		exit(1);
	}
	ioctl(fd, DAHDI_GETEVENT);

	i = DAHDI_FLUSH_ALL;
	if (ioctl(fd,DAHDI_FLUSH,&i) == -1)
	   {
		perror("DAHDI_FLUSH");
		exit(255);
	   }

	i = 0;
	if(argc==3){
		timeout=atoi(argv[2]);
		start_time=time(NULL);
		printf("Using Timeout of %d Seconds\n",timeout);
	}

	for(;;) {
		res = bs;
		startbyte = 0;

		for (x=0;x<bs;x++) 
			outbuf[x] = c1++;

		if (!skipcount) {
			if (!sign_packet(outbuf, bs))
			{
				printf ("Out buffer too short. Abort\n");
				exit(1);
			}
		}

		res = write(fd,outbuf,bs);
		if (res != bs)
		   {
			printf("Res is %d: %s\n", res, strerror(errno));
			ioctl(fd, DAHDI_GETEVENT, &x);
			printf("Event: %d\n", x);
			exit(1);
		}

		res = read(fd, inbuf, bs);
		if (res < bs) {
			printf("Res is %d\n", res);
			exit(1);
		}


		if (!signaturefound) {
			startbyte = search_signature(inbuf, bs);
			if (startbyte == -1) {

				printf("Flushing the %d invalid packet\n", skipcount);
				skipcount++;
				continue;
			} else {
				printf("Going for it...\n");
				signaturefound++;
				skipcount++;
			}
		}

		if (!setup) {
			c = inbuf[startbyte];
			setup++;
		}
		for (x=startbyte;x<bs;x++)  {
			if (inbuf[x] != c) {
				printf("(Error %d): Unexpected result, %d != %d, %d bytes since last error.\n", ++errors, inbuf[x],c, bytes);
				c = inbuf[x];
				bytes=0;
			}
			c++;
			bytes++;
		}

		print_packet(inbuf,bs); /* To be deleted */

		packets++;
		if (!(packets % 16))
			printf(".");

		if(timeout && (time(NULL)-start_time)>timeout){
			printf("Timeout achieved Ending Program\n");
			return errors;
		}
	}
	
}
